// Cube-Dijkstra-26x26-FB

#include <iostream>
#include <string>
#include <stack>
using namespace std;

const int xsize = 26;
const int ysize = 26;
const int inf = 1000000;

int board[xsize][ysize];
    
struct cube
{ int front,back,top,right,down,left;

  cube(int f=0,int b=0,int t=0,int r=0,int d=0,int l=0)
  { front = f; back = b; top = t; right = r; down = d; left = l; }
  
  cube move(int dir)
  { if(dir==0) return cube(down,top,front,right,back,left); // north
    if(dir==1) return cube(front,back,left,top,right,down); // east 
    if(dir==2) return cube(top,down,back,right,front,left); // south     
    return            cube(front,back,right,down,left,top); // west  
  }   
  
  bool operator==(const cube & c) const
  { return front==c.front && back==c.back &&
           top==c.top && right==c.right &&        
           down==c.down && left==c.left; 
  }          
};

cube c[24];
int t[24][4];
int m;

int rot(cube cb)
{ int k=0;
  while(k<m && !(cb == c[k])) k++;
  if(k==m)  
  { c[k]=cb; m++; 
    for(int i=0; i<4; i++)
      t[k][i] = rot(cb.move(i));  
  }    
  return k;  
}    
  
struct vertex 
{ int x, y, p; 
  vertex() { x = y = p = 0; }
  vertex(int _x, int _y, int _p) 
  { x = _x; y = _y; p = _p; }
};

vertex a[xsize*ysize*24];
vertex pr[xsize][ysize][24];
int d[xsize][ysize][24];
int s[xsize][ysize][24];

int dijkstra(int x1,int y1,int x2,int y2)
{ const int dx[4] = {0, 1,  0, -1 };
  const int dy[4] = {1, 0, -1,  0 };

  int n=0;
  for(int x=0; x<xsize; x++)
    for(int y=0; y<ysize; y++)
      if(board[x][y]==0)
      for(int p=0; p<m; p++)
      { a[n].x=x; a[n].y=y; a[n].p=p; n++;
        d[x][y][p] = inf;
        s[x][y][p] = inf;
      }

  d[x1][y1][0]=0;       
  s[x1][y1][0]=c[0].down;  
  
  while(n>0)
  { int k=-1, dmin=inf, smin=inf;
    for(int i=0; i<n; i++)
    { int x=a[i].x, y=a[i].y, p=a[i].p;
      if(dmin > d[x][y][p] || 
         (dmin == d[x][y][p] && smin > s[x][y][p]))
      { k = i; dmin = d[x][y][p]; smin = s[x][y][p]; }
    } 
    if(k==-1) return -1; 
         
    vertex u = a[k];
    a[k] = a[n-1]; n--;
                  
    if(u.x==x2 && u.y==y2) return u.p; 
    
    for(int j=0; j<4; j++)
    { vertex v;
      v.x = u.x + dx[j]; v.y = u.y + dy[j]; v.p=t[u.p][j];
      if(0<=v.x && v.x<xsize && 0<=v.y && v.y<ysize && board[v.x][v.y]==0)
      {
        if( d[u.x][u.y][u.p]+1 < d[v.x][v.y][v.p] ||
            ( d[u.x][u.y][u.p]+1 == d[v.x][v.y][v.p] && 
              s[u.x][u.y][u.p]+c[v.p].down < s[v.x][v.y][v.p]
            )
          ) 
          { d[v.x][v.y][v.p] = d[u.x][u.y][u.p] + 1;
            s[v.x][v.y][v.p] = s[u.x][u.y][u.p]+c[v.p].down;
            pr[v.x][v.y][v.p] = vertex(u.x,u.y,u.p);
          }
      }
    }    
  }
}       
  
void showpath(int x1,int y1,int x2,int y2, int p)
{ board[x1][y1]=board[x2][y2]=2; 

  if(p>=0) 
  { int dmin = d[x2][y2][p];
    int smin = s[x2][y2][p];

    stack<vertex> path;
    vertex u(x2,y2,p);
    path.push(u);
    for(int i=dmin; i>0; i--)
    { u = pr[u.x][u.y][u.p];
      path.push(u);
    }  
  
    while(! path.empty())
    { vertex u = path.top(); 
      path.pop();
      board[u.x][u.y]=2;
      cout << char('a'+u.x) << 1+u.y << "/" << c[u.p].down << " ";
    }
    cout << endl;
    cout << "dmin=" << dmin << "  smin=" << smin << endl;
  }  
 
    cout << "     ";
    for(int x=0; x<xsize; x++)
      cout << char('a' + x) << " ";
    cout << endl;
    cout << "     ---------------------------------------------------" << endl;
    for(int y = ysize-1; y>=0; y--)
    { cout << 1+y << "   ";
      if(1+y<10) cout << " ";
      for(int x=0; x<xsize; x++)
      { if(board[x][y]==0) cout << ". ";
        if(board[x][y]==1) cout << "X ";
        if(board[x][y]==2) cout << "o ";
      }
      cout << "  " << 1+y << endl;
    }
    cout << "     ---------------------------------------------------" << endl;
    cout << "     ";
    for(int x=0; x<xsize; x++)
      cout << char('a' + x) << " ";
    cout << endl;
}

int main()
{ int x1,y1,x2,y2;
  char ch;
  cin >> ch >> y1; x1=ch-'a'; y1--;
  cin >> ch >> y2; x2=ch-'a'; y2--;

  int side[6];
  for(int i=0; i<6; i++)
    cin >> side[i];
  rot(cube(side[0],side[1],side[2],side[3],side[4],side[5]));
   
  int fig;  
  cin >> fig;
  for(int i=0; i<fig; i++)
  { char ch; 
    int y;
    cin >> ch >> y;
    int x = ch-'a';
    y--;
    board[x][y] = 1;
  }   

  int p = dijkstra(x1,y1,x2,y2);
  
  if(p>=0) 
    cout << s[x2][y2][p] << endl;
  else 
    cout << -1 << endl;
         
//  showpath(x1,y1,x2,y2,p);
   
  return 0;
}
      
